@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20767 $
- * $Author: rjongbloed $
- * $Date: 2008-08-29 08:44:24 +0200 (ven 29 aoû 2008) $
+ * $Revision: 20867 $
+ * $Author: dsandras $
+ * $Date: 2008-09-05 11:11:56 +0000 (Fri, 05 Sep 2008) $
  */
 
 #ifndef __OPAL_MANAGER_H
@@ -953,13 +953,13 @@ class OpalManager : public PObject
       const PIPSocket::Address & address
     );
 
-    /**Return the STUN server to use.
+    /**Return the NAT method to use.
        Returns NULL if address is a local address as per IsLocalAddress().
-       Always returns the STUN server if address is zero.
+       Always returns the NAT method if address is zero.
        Note, the pointer is NOT to be deleted by the user.
       */
-    PSTUNClient * GetSTUN(
-      const PIPSocket::Address & address = 0
+    virtual PNatMethod * GetNatMethod(
+      const PIPSocket::Address & remoteAddress = PIPSocket::GetDefaultIpAny()
     ) const;
 
     /**Set the STUN server address, is of the form host[:port]
@@ -974,6 +974,10 @@ class OpalManager : public PObject
       */
     const PString & GetSTUNServer() const { return stunServer; }
 
+    /**Return the STUN client instance in use.
+      */
+    PSTUNClient * GetSTUNClient() const { return stun; }
+
     /**Get the TCP port number base for H.245 channels
      */
     WORD GetTCPPortBase() const { return tcpPorts.base; }
