@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20722 $
+ * $Revision: 21035 $
  * $Author: rjongbloed $
- * $Date: 2008-08-22 08:33:16 +0200 (ven 22 aoû 2008) $
+ * $Date: 2008-09-17 07:57:58 +0000 (Wed, 17 Sep 2008) $
  */
 
 #ifndef __OPAL_RTP_H
@@ -39,8 +39,8 @@
 #endif
 
 #include <opal/buildopts.h>
-#include <ptlib/sockets.h>
 
+#include <ptlib/sockets.h>
 
 class RTP_JitterBuffer;
 class PNatMethod;
@@ -287,7 +287,7 @@ class RTP_Session;
 
 ///////////////////////////////////////////////////////////////////////////////
 
-#ifdef OPAL_STATISTICS
+#if OPAL_STATISTICS
 
 /**This class carries statistics on the media stream.
   */
@@ -368,6 +368,13 @@ class RTP_UserData : public PObject
       const RTP_Session & session   ///<  Session with statistics
     ) const;
 #endif
+
+    /**Callback from the RTP session when a call needs to be shut down.
+       The default behaviour does nothing.
+      */
+    virtual void OnClearCall(
+      const RTP_Session & session   ///<  Session with statistics
+    );
 };
 
 class RTP_Encoding;
@@ -382,16 +389,33 @@ class RTP_Session : public PObject
   public:
   /**@name Construction */
   //@{
+    struct Params {
+      Params()
+        : id(0)
+#if OPAL_RTP_AGGREGATE
+        , aggregator(NULL)
+#endif
+        , userData(NULL)
+        , autoDelete(true)
+        , isAudio(false)
+        , remoteIsNAT(false)
+      { }
+
+      PString             encoding;    ///<  identifies initial RTP encoding (RTP/AVP, UDPTL etc)
+      unsigned            id;          ///<  Session ID for RTP channel
+#if OPAL_RTP_AGGREGATE
+      PHandleAggregator * aggregator;  ///<  RTP aggregator
+#endif
+      RTP_UserData      * userData;    ///<  Optional data for session.
+      bool                autoDelete;  ///<  Delete optional data with session.
+      bool                isAudio;     ///<  is audio RTP data
+      bool                remoteIsNAT; ///<  Remote is behid NAT
+    };
+
     /**Create a new RTP session.
      */
     RTP_Session(
-      const PString & encoding,            ///<  identifies initial RTP encoding (RTP/AVP, UDPTL etc)
-#if OPAL_RTP_AGGREGATE
-      PHandleAggregator * aggregator,      ///<  RTP aggregator
-#endif
-      unsigned id,                         ///<  Session ID for RTP channel
-      RTP_UserData * userData = NULL,      ///<  Optional data for session.
-      PBoolean autoDeleteUserData = PTrue  ///<  Delete optional data with session.
+      const Params & options ///< Parameters to construct with session.
     );
 
     /**Delete a session.
@@ -492,7 +516,7 @@ class RTP_Session : public PObject
       */
     virtual PString GetLocalHostName() = 0;
 
-#ifdef OPAL_STATISTICS
+#if OPAL_STATISTICS
     virtual void GetStatistics(OpalMediaStatistics & statistics, bool receiver) const;
 #endif
   //@}
@@ -902,12 +926,7 @@ class RTP_UDP : public RTP_Session
     /**Create a new RTP channel.
      */
     RTP_UDP(
-      const PString & _format,
-#if OPAL_RTP_AGGREGATE
-      PHandleAggregator * aggregator, ///< RTP aggregator
-#endif
-      unsigned id,                    ///<  Session ID for RTP channel
-      PBoolean remoteIsNAT                ///<  PTrue is remote is behind NAT
+      const Params & options ///< Parameters to construct with session.
     );
 
     /// Destroy the RTP
@@ -1079,6 +1098,7 @@ class RTP_UDP : public RTP_Session
     bool remoteIsNAT;
     bool localHasNAT;
     bool first;
+    int  badTransmitCounter;
 };
 
 /////////////////////////////////////////////////////////////////////////////
@@ -1121,12 +1141,7 @@ class SecureRTP_UDP : public RTP_UDP
     /**Create a new RTP channel.
      */
     SecureRTP_UDP(
-      const PString & encoding,       ///<  identifies initial RTP encoding (RTP/AVP, UDPTL etc)
-#if OPAL_RTP_AGGREGATE
-      PHandleAggregator * aggregator, ///< RTP aggregator
-#endif
-      unsigned id,                    ///<  Session ID for RTP channel
-      PBoolean remoteIsNAT                ///<  PTrue is remote is behind NAT
+      const Params & options ///< Parameters to construct with session.
     );
 
     /// Destroy the RTP
