@@ -24,15 +24,16 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Log: sip.h,v $
- * Revision 2.1  2002/02/01 04:53:01  robertj
- * Added (very primitive!) SIP support.
- *
+ * $Revision: 21004 $
+ * $Author: rjongbloed $
+ * $Date: 2008-09-16 07:08:56 +0000 (Tue, 16 Sep 2008) $
  */
 
 #ifndef __OPAL_SIP_H
 #define __OPAL_SIP_H
 
+#include <opal/buildopts.h>
+
 #include <sip/sipep.h>
 #include <sip/sipcon.h>
 
