@@ -36,9 +36,9 @@
  * Contributor(s): Guilhem Tardy (gtardy@salyens.com)
  *                 Craig Southeren (craigs@postincrement.com)
  *
- * $Revision: 20629 $
- * $Author: rjongbloed $
- * $Date: 2008-07-28 09:57:33 +0200 (lun 28 jui 2008) $
+ * $Revision: 20942 $
+ * $Author: ms30002000 $
+ * $Date: 2008-09-12 17:44:53 +0000 (Fri, 12 Sep 2008) $
  */
 
 /*
@@ -57,6 +57,9 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#ifdef SOLARIS
+#include <alloca.h>
+#endif
 
 #ifndef PLUGIN_CODEC_DLL_EXPORTS
 #include "plugin-config.h"
