@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20712 $
- * $Author: dereksmithies $
- * $Date: 2008-08-21 05:20:32 +0200 (jeu 21 aoû 2008) $
+ * $Revision: 21110 $
+ * $Author: rjongbloed $
+ * $Date: 2008-09-21 23:56:12 +0000 (Sun, 21 Sep 2008) $
  */
 
 #include <ptlib.h>
@@ -179,7 +179,7 @@ OpalConnection::OpalConnection(OpalCall & call,
   , silenceDetector(NULL)
   , echoCanceler(NULL)
   , stringOptions((_stringOptions == NULL) ? NULL : new OpalConnection::StringOptions(*_stringOptions))
-#ifdef OPAL_STATISTICS
+#if OPAL_STATISTICS
   , m_VideoUpdateRequestsSent(0)
 #endif
 {
@@ -978,7 +978,7 @@ PBoolean OpalConnection::PromptUserInput(PBoolean /*play*/)
 }
 
 
-#if P_DTMF
+#if OPAL_PTLIB_DTMF
 void OpalConnection::OnUserInputInBandDTMF(RTP_DataFrame & frame, INT)
 {
   // This function is set up as an 'audio filter'.
@@ -1043,6 +1043,17 @@ PString OpalConnection::GetCalledPartyURL()
 }
 
 
+void OpalConnection::CopyPartyNames(const OpalConnection & other)
+{
+  remotePartyName     = other.remotePartyName;
+  remotePartyNumber   = other.remotePartyNumber;
+  remotePartyAddress  = other.remotePartyAddress;
+  m_calledPartyName   = other.m_calledPartyName;
+  m_calledPartyNumber = other.m_calledPartyNumber;
+  remoteProductInfo   = other.remoteProductInfo;
+}
+
+
 void OpalConnection::SetAudioJitterDelay(unsigned minDelay, unsigned maxDelay)
 {
   maxDelay = PMAX(10, PMIN(maxDelay, 999));
