@@ -28,9 +28,9 @@
  *     US Joint Forces Command Joint Concept Development & Experimentation (J9)
  *     http://www.jfcom.mil/about/abt_j9.htm
  *
- * $Revision: 20547 $
+ * $Revision: 21115 $
  * $Author: rjongbloed $
- * $Date: 2008-07-08 04:51:59 +0200 (mar 08 jui 2008) $
+ * $Date: 2008-09-22 03:00:58 +0000 (Mon, 22 Sep 2008) $
  */
 
 #include <ptlib.h>
@@ -39,8 +39,9 @@
 #pragma implementation "transports.h"
 #endif
 
-#include <opal/transports.h>
+#include <opal/buildopts.h>
 
+#include <opal/transports.h>
 #include <opal/manager.h>
 #include <opal/endpoint.h>
 #include <opal/call.h>
@@ -57,7 +58,7 @@ static PFactory<OpalInternalTransport>::Worker<OpalInternalTCPTransport> opalInt
 static PFactory<OpalInternalTransport>::Worker<OpalInternalTCPTransport>  opalInternalIPTransportFactory(IpPrefix, true);
 static PFactory<OpalInternalTransport>::Worker<OpalInternalUDPTransport> opalInternalUDPTransportFactory(UdpPrefix, true);
 
-#if P_SSL
+#if OPAL_PTLIB_SSL
 #include <ptclib/pssl.h>
 static const char TcpsPrefix[] = "tcps$";
 static PFactory<OpalInternalTransport>::Worker<OpalInternalTCPSTransport> opalInternalTCPSTransportFactory(TcpsPrefix, true);
@@ -135,12 +136,12 @@ PBoolean OpalTransportAddress::IsCompatible(const OpalTransportAddress & address
   PCaselessString theirPrefix = address.Left(address.Find('$'));
   return myPrefix == theirPrefix ||
         (myPrefix    == IpPrefix && (theirPrefix == TcpPrefix || theirPrefix == UdpPrefix 
-#if P_SSL
+#if OPAL_PTLIB_SSL
                                      || theirPrefix == TcpsPrefix
 #endif
                                     )) ||
         (theirPrefix == IpPrefix && (myPrefix    == TcpPrefix || myPrefix    == UdpPrefix 
-#if P_SSL
+#if OPAL_PTLIB_SSL
                                      || myPrefix    == TcpsPrefix
 #endif
                                     ));
@@ -267,45 +268,48 @@ PBoolean OpalInternalTransport::GetIpAndPort(const OpalTransportAddress &,
 
 //////////////////////////////////////////////////////////////////////////
 
-static PBoolean SplitAddress(const PString & addr, PString & host, PString & service)
+static PBoolean SplitAddress(const PString & addr, PString & host, PString & device, PString & service)
 {
   // skip transport identifier
   PINDEX dollar = addr.Find('$');
   if (dollar == P_MAX_INDEX)
-    return PFalse;
+    return false;
   
-  PINDEX lastChar = addr.GetLength()-1;
-  if (addr[lastChar] == '+')
-    lastChar--;
+  PINDEX length = addr.GetLength();
+  if (addr[length-1] == '+')
+    length--;
 
-  PINDEX percent = addr.FindLast('%');
-  if (percent != P_MAX_INDEX)
-    lastChar = percent-1;
-    
-  PINDEX bracket = addr.FindLast(']', lastChar);
+  PINDEX bracket = addr.FindLast(']');
   if (bracket == P_MAX_INDEX)
     bracket = 0;
 
-  PINDEX colon = addr.Find(':', bracket);
-  if (colon >= lastChar)
-    host = addr(dollar+1, lastChar);
-  else {
-    host = addr(dollar+1, colon-1);
-    service = addr(colon+1, lastChar);
-  }
+  PINDEX colon = addr.FindLast(':');
+  if (colon == P_MAX_INDEX || colon < bracket)
+    colon = length;
 
-  return PTrue;
+  PINDEX percent = addr.Find('%', dollar);
+  if (percent == P_MAX_INDEX)
+    percent = colon;
+
+  host = addr(dollar+1, percent-1);
+  device = addr(percent, colon-1);
+  service = addr(colon+1, length-1);
+
+  return true;
 }
 
 
 PString OpalInternalIPTransport::GetHostName(const OpalTransportAddress & address) const
 {
-  PString host, service;
-  if (!SplitAddress(address, host, service))
+  PString host, device, service;
+  if (!SplitAddress(address, host, device, service))
     return address;
 
+  if (!device.IsEmpty())
+    return host+device;
+
   PIPSocket::Address ip;
-  if (PIPSocket::GetHostAddress(host, ip))
+  if (ip.FromString(host))
     return ip.AsString();
 
   return host;
@@ -316,11 +320,11 @@ PBoolean OpalInternalIPTransport::GetIpAndPort(const OpalTransportAddress & addr
                                            PIPSocket::Address & ip,
                                            WORD & port) const
 {
-  PString host, service;
-  if (!SplitAddress(address, host, service))
+  PString host, device, service;
+  if (!SplitAddress(address, host, device, service))
     return PFalse;
 
-  if (host.IsEmpty()) {
+  if (host.IsEmpty() && device.IsEmpty()) {
     PTRACE(2, "Opal\tIllegal IP transport address: \"" << address << '"');
     return PFalse;
   }
@@ -345,10 +349,17 @@ PBoolean OpalInternalIPTransport::GetIpAndPort(const OpalTransportAddress & addr
     return PTrue;
   }
 
-  if (PIPSocket::GetHostAddress(host, ip))
-    return PTrue;
+  if (device.IsEmpty()) {
+    if (PIPSocket::GetHostAddress(host, ip))
+      return true;
+    PTRACE(1, "Opal\tCould not find host \"" << host << '"');
+  }
+  else {
+    if (ip.FromString(device))
+      return true;
+    PTRACE(1, "Opal\tCould not find device \"" << device << '"');
+  }
 
-  PTRACE(1, "Opal\tCould not find host : \"" << host << '"');
   return PFalse;
 }
 
@@ -472,12 +483,13 @@ PBoolean OpalListener::StartThread(const PNotifier & theAcceptHandler, ThreadMod
 
 OpalTransportAddressArray OpalGetInterfaceAddresses(const OpalListenerList & listeners,
                                                     PBoolean excludeLocalHost,
-                                                    OpalTransport * associatedTransport)
+                                                    OpalTransport * associatedTransport,
+                                                    PNatMethod * natMethod)
 {
   OpalTransportAddressArray interfaceAddresses;
 
   for (OpalListenerList::const_iterator listener = listeners.begin(); listener != listeners.end(); ++listener) {
-    OpalTransportAddressArray newAddrs = OpalGetInterfaceAddresses(listener->GetTransportAddress(), excludeLocalHost, associatedTransport);
+    OpalTransportAddressArray newAddrs = OpalGetInterfaceAddresses(listener->GetTransportAddress(), excludeLocalHost, associatedTransport, natMethod);
     PINDEX size  = interfaceAddresses.GetSize();
     PINDEX nsize = newAddrs.GetSize();
     interfaceAddresses.SetSize(size + nsize);
@@ -490,40 +502,56 @@ OpalTransportAddressArray OpalGetInterfaceAddresses(const OpalListenerList & lis
 }
 
 
+static void AddTransportAddress(OpalTransportAddressArray & addresses, const PIPSocket::Address & ip, WORD port, PNatMethod * natMethod)
+{
+  addresses.Append(new OpalTransportAddress(ip, port));
+
+  if (natMethod == NULL)
+    return;
+
+  PIPSocket::Address natIP;
+  if (!natMethod->GetInterfaceAddress(natIP))
+    return;
+
+  if (natIP != ip)
+    return;
+
+  if (natMethod->GetExternalAddress(natIP))
+    addresses.Append(new OpalTransportAddress(natIP, port));
+}
+
+
 OpalTransportAddressArray OpalGetInterfaceAddresses(const OpalTransportAddress & addr,
                                                     PBoolean excludeLocalHost,
-                                                    OpalTransport * associatedTransport)
+                                                    OpalTransport * associatedTransport,
+                                                    PNatMethod * natMethod)
 {
+  OpalTransportAddressArray interfaceAddresses;
+  PIPSocket::InterfaceTable interfaces;
+
   PIPSocket::Address ip;
   WORD port = 0;
-  if (!addr.GetIpAndPort(ip, port) || !ip.IsAny())
-    return addr;
+  if (!addr.GetIpAndPort(ip, port) || !ip.IsAny() || !PIPSocket::GetInterfaceTable(interfaces))
+    AddTransportAddress(interfaceAddresses, ip, port, natMethod);
+  else {
+    PINDEX i;
+    PIPSocket::Address firstAddress(0);
 
-  PIPSocket::InterfaceTable interfaces;
-  if (!PIPSocket::GetInterfaceTable(interfaces))
-    return addr;
-
-  if (interfaces.GetSize() == 1)
-    return OpalTransportAddress(interfaces[0].GetAddress(), port);
-
-  PINDEX i;
-  OpalTransportAddressArray interfaceAddresses;
-  PIPSocket::Address firstAddress(0);
-
-  if (associatedTransport != NULL) {
-    if (associatedTransport->GetLocalAddress().GetIpAddress(firstAddress)) {
-      for (i = 0; i < interfaces.GetSize(); i++) {
-        PIPSocket::Address ip = interfaces[i].GetAddress();
-        if (ip == firstAddress)
-          interfaceAddresses.Append(new OpalTransportAddress(ip, port));
+    if (associatedTransport != NULL) {
+      if (associatedTransport->GetLocalAddress().GetIpAddress(firstAddress)) {
+        for (i = 0; i < interfaces.GetSize(); i++) {
+          PIPSocket::Address ip = interfaces[i].GetAddress();
+          if (ip == firstAddress)
+            AddTransportAddress(interfaceAddresses, ip, port, natMethod);
+        }
       }
     }
-  }
 
-  for (i = 0; i < interfaces.GetSize(); i++) {
-    PIPSocket::Address ip = interfaces[i].GetAddress();
-    if (ip != firstAddress && !(excludeLocalHost && ip.IsLoopback()))
-      interfaceAddresses.Append(new OpalTransportAddress(ip, port));
+    for (i = 0; i < interfaces.GetSize(); i++) {
+      PIPSocket::Address ip = interfaces[i].GetAddress();
+      if (ip != firstAddress && !(excludeLocalHost && ip.IsLoopback()))
+        AddTransportAddress(interfaceAddresses, ip, port, natMethod);
+    }
   }
 
   return interfaceAddresses;
@@ -699,7 +727,7 @@ OpalTransport * OpalListenerTCP::CreateTransport(const OpalTransportAddress & lo
   if (myLocalAddress.IsCompatible(remoteAddress)) {
     if (!localAddress.IsEmpty())
       return localAddress.CreateTransport(endpoint, OpalTransportAddress::NoBinding);
-#ifdef P_SSL
+#if OPAL_PTLIB_SSL
     if (remoteAddress.NumCompare(TcpsPrefix) == EqualTo)
       return new OpalTransportTCPS(endpoint);
 #endif
@@ -722,7 +750,7 @@ OpalListenerUDP::OpalListenerUDP(OpalEndPoint & endpoint,
                                  WORD port,
                                  PBoolean exclusive)
   : OpalListenerIP(endpoint, binding, port, exclusive),
-    listenerBundle(PMonitoredSockets::Create(binding.AsString(), !exclusive, endpoint.GetManager().GetSTUN()))
+    listenerBundle(PMonitoredSockets::Create(binding.AsString(), !exclusive, endpoint.GetManager().GetNatMethod()))
 {
 }
 
@@ -731,7 +759,7 @@ OpalListenerUDP::OpalListenerUDP(OpalEndPoint & endpoint,
                                  const OpalTransportAddress & binding,
                                  OpalTransportAddress::BindOptions option)
   : OpalListenerIP(endpoint, binding, option),
-    listenerBundle(PMonitoredSockets::Create(binding.GetHostName(), !exclusiveListener, endpoint.GetManager().GetSTUN()))
+    listenerBundle(PMonitoredSockets::Create(binding.GetHostName(), !exclusiveListener, endpoint.GetManager().GetNatMethod()))
 {
 }
 
@@ -1183,18 +1211,15 @@ const char * OpalTransportTCP::GetProtoPrefix() const
 OpalTransportUDP::OpalTransportUDP(OpalEndPoint & ep,
                                    PIPSocket::Address binding,
                                    WORD localPort,
-                                   PBoolean reuseAddr)
+                                   bool reuseAddr,
+                                   bool preOpen)
   : OpalTransportIP(ep, binding, localPort)
   , manager(ep.GetManager())
 {
-  PMonitoredSockets * sockets = PMonitoredSockets::Create(binding.AsString(), reuseAddr);
-  if (sockets->Open(localPort)) {
-    Open(new PMonitoredSocketChannel(sockets, PFalse));
-    PTRACE(3, "OpalUDP\tBinding to interface: " << localAddress << ':' << localPort);
-  }
-  else {
-    PTRACE(1, "OpalUDP\tCould not bind to interface: " << localAddress << ':' << localPort);
-  }
+  PMonitoredSockets * sockets = PMonitoredSockets::Create(binding.AsString(), reuseAddr, manager.GetNatMethod());
+  if (preOpen)
+    sockets->Open(localPort);
+  Open(new PMonitoredSocketChannel(sockets, PFalse));
 }
 
 
@@ -1263,8 +1288,6 @@ PBoolean OpalTransportUDP::Connect()
 
   OpalManager & manager = endpoint.GetManager();
 
-  bundle->SetNatMethod(manager.GetSTUN(remoteAddress));
-
   localPort = manager.GetNextUDPPort();
   WORD firstPort = localPort;
   while (!bundle->Open(localPort)) {
@@ -1456,7 +1479,7 @@ const char * OpalTransportUDP::GetProtoPrefix() const
 
 //////////////////////////////////////////////////////////////////////////
 
-#if P_SSL
+#if OPAL_PTLIB_SSL
 
 static PBoolean SetSSLCertificate(PSSLContext & sslContext,
                              const PFilePath & certificateFile,
